@@ -14,9 +14,11 @@ import tempfile
 import zipfile
 
 from base64 import urlsafe_b64encode
+from functools import cached_property
 from io import StringIO
 from pathlib import Path
 from typing import TYPE_CHECKING
+from typing import Any
 from typing import TextIO
 
 import packaging.tags
@@ -38,6 +40,8 @@ if TYPE_CHECKING:
 
     from poetry.core.poetry import Poetry
 
+    ZipInfoTimestamp = tuple[int, int, int, int, int, int]
+
 wheel_file_template = """\
 Wheel-Version: 1.0
 Generator: poetry-core {version}
@@ -58,8 +62,9 @@ class WheelBuilder(Builder):
         executable: Path | None = None,
         editable: bool = False,
         metadata_directory: Path | None = None,
+        config_settings: dict[str, Any] | None = None,
     ) -> None:
-        super().__init__(poetry, executable=executable)
+        super().__init__(poetry, executable=executable, config_settings=config_settings)
 
         self._records: list[tuple[str, str, int]] = []
         self._original_path = self._path
@@ -77,13 +82,15 @@ class WheelBuilder(Builder):
         executable: Path | None = None,
         editable: bool = False,
         metadata_directory: Path | None = None,
+        config_settings: dict[str, Any] | None = None,
     ) -> str:
-        wb = WheelBuilder(
+        wb = cls(
             poetry,
             original=original,
             executable=executable,
             editable=editable,
             metadata_directory=metadata_directory,
+            config_settings=config_settings,
         )
         wb.build(target_dir=directory)
 
@@ -102,17 +109,21 @@ class WheelBuilder(Builder):
 
         target_dir = target_dir or self.default_target_dir
         if not target_dir.exists():
-            target_dir.mkdir()
+            target_dir.mkdir(parents=True)
 
-        fd, temp_path = tempfile.mkstemp(suffix=".whl")
+        fd, temp = tempfile.mkstemp(suffix=".whl")
 
-        st_mode = os.stat(temp_path).st_mode
+        temp_path = Path(temp)
+        st_mode = temp_path.stat().st_mode
         new_mode = normalize_file_permissions(st_mode)
-        os.chmod(temp_path, new_mode)
+        temp_path.chmod(new_mode)
 
-        with os.fdopen(fd, "w+b") as fd_file, zipfile.ZipFile(
-            fd_file, mode="w", compression=zipfile.ZIP_DEFLATED
-        ) as zip_file:
+        with (
+            os.fdopen(fd, "w+b") as fd_file,
+            zipfile.ZipFile(
+                fd_file, mode="w", compression=zipfile.ZIP_DEFLATED
+            ) as zip_file,
+        ):
             if self._editable:
                 self._build(zip_file)
                 self._add_pth(zip_file)
@@ -137,7 +148,7 @@ class WheelBuilder(Builder):
         wheel_path = target_dir / self.wheel_filename
         if wheel_path.exists():
             wheel_path.unlink()
-        shutil.move(temp_path, str(wheel_path))
+        shutil.move(str(temp_path), str(wheel_path))
 
         logger.info(f"Built {self.wheel_filename}")
         return wheel_path
@@ -209,13 +220,40 @@ class WheelBuilder(Builder):
                         self._add_file(wheel, pkg, rel_path)
 
     def _get_build_purelib_dir(self) -> Path:
-        return self._path / "build/lib"
+        return self._path / "build" / "lib"
 
     def _get_build_platlib_dir(self) -> Path:
         # Roughly equivalent to the naming convention in used by distutils, see:
         # distutils.command.build.build.finalize_options
-        plat_specifier = f"{sysconfig.get_platform()}-{sys.implementation.cache_tag}"
-        return self._path / f"build/lib.{plat_specifier}"
+        if self.executable != Path(sys.executable):
+            # poetry-core is not run in the build environment
+            # -> this is probably not a PEP 517 build but a poetry build
+            try:
+                output = subprocess.check_output(
+                    [
+                        self.executable.as_posix(),
+                        "-c",
+                        """
+import sysconfig
+import sys
+print(sysconfig.get_platform(), sys.implementation.cache_tag, sep='-')
+""",
+                    ],
+                    stderr=subprocess.STDOUT,
+                    text=True,
+                    encoding="utf-8",
+                )
+            except subprocess.CalledProcessError as e:
+                raise RuntimeError(
+                    "Failed to get build_platlib_dir for python interpreter"
+                    f" '{self.executable.as_posix()}':\n{e.output}"
+                )
+            plat_specifier = output.strip()
+        else:
+            plat_specifier = "-".join(
+                (sysconfig.get_platform(), sys.implementation.cache_tag)
+            )
+        return self._path / "build" / f"lib.{plat_specifier}"
 
     def _get_build_lib_dir(self) -> Path | None:
         # Either the purelib or platlib path will have been used when building
@@ -239,23 +277,27 @@ class WheelBuilder(Builder):
 
     def _run_build_command(self, setup: Path) -> None:
         if self._editable:
-            subprocess.check_call([
+            subprocess.check_call(
+                [
+                    self.executable.as_posix(),
+                    str(setup),
+                    "build_ext",
+                    "--inplace",
+                ]
+            )
+        subprocess.check_call(
+            [
                 self.executable.as_posix(),
                 str(setup),
-                "build_ext",
-                "--inplace",
-            ])
-        subprocess.check_call([
-            self.executable.as_posix(),
-            str(setup),
-            "build",
-            "-b",
-            str(self._path / "build"),
-            "--build-purelib",
-            str(self._get_build_purelib_dir()),
-            "--build-platlib",
-            str(self._get_build_platlib_dir()),
-        ])
+                "build",
+                "-b",
+                str(self._path / "build"),
+                "--build-purelib",
+                str(self._get_build_purelib_dir()),
+                "--build-platlib",
+                str(self._get_build_platlib_dir()),
+            ]
+        )
 
     def _run_build_script(self, build_script: str) -> None:
         logger.debug(f"Executing build script: {build_script}")
@@ -273,10 +315,7 @@ class WheelBuilder(Builder):
         dist_info = metadata_directory / self.dist_info
         dist_info.mkdir(parents=True, exist_ok=True)
 
-        if (
-            "scripts" in self._poetry.local_config
-            or "plugins" in self._poetry.local_config
-        ):
+        if self._poetry.package.entry_points:
             with (dist_info / "entry_points.txt").open(
                 "w", encoding="utf-8", newline="\n"
             ) as f:
@@ -293,7 +332,7 @@ class WheelBuilder(Builder):
                 logger.debug(f"Skipping: {legal_file.as_posix()}")
                 continue
 
-            dest = dist_info / legal_file.relative_to(self._path)
+            dest = dist_info / "licenses" / legal_file.relative_to(self._path)
             dest.parent.mkdir(parents=True, exist_ok=True)
             shutil.copy(legal_file, dest)
 
@@ -417,7 +456,7 @@ for t in packaging_tags.sys_tags():
     ) -> None:
         # We always want to have /-separated paths in the zip file and in RECORD
         rel_path_name = rel_path.as_posix()
-        zinfo = zipfile.ZipInfo(rel_path_name)
+        zinfo = zipfile.ZipInfo(rel_path_name, self._zipfile_date_time)
 
         # Normalize permission bits to either 755 (executable) or 644
         st_mode = full_path.stat().st_mode
@@ -450,10 +489,7 @@ for t in packaging_tags.sys_tags():
         sio = StringIO()
         yield sio
 
-        # The default is a fixed timestamp rather than the current time, so
-        # that building a wheel twice on the same computer can automatically
-        # give you the exact same result.
-        date_time = (2016, 1, 1, 0, 0, 0)
+        date_time = self._zipfile_date_time
         zi = zipfile.ZipInfo(rel_path, date_time)
         zi.external_attr = (0o644 & 0xFFFF) << 16  # Unix attributes
         b = sio.getvalue().encode("utf-8")
@@ -463,6 +499,40 @@ for t in packaging_tags.sys_tags():
         wheel.writestr(zi, b, compress_type=zipfile.ZIP_DEFLATED)
         self._records.append((rel_path, hash_digest, len(b)))
 
+    @cached_property
+    def _zipfile_date_time(self) -> ZipInfoTimestamp:
+        import time
+
+        # The default is a fixed timestamp rather than the current time, so
+        # that building a wheel twice on the same computer can automatically
+        # give you the exact same result.
+        default = (2016, 1, 1, 0, 0, 0)
+        try:
+            _env_date = time.gmtime(int(os.environ["SOURCE_DATE_EPOCH"]))[:6]
+        except ValueError:
+            logger.warning(
+                "SOURCE_DATE_EPOCH environment variable value"
+                " is not an int, setting zipinfo date to default=%s",
+                default,
+            )
+            return default
+        except KeyError:
+            logger.debug(
+                "SOURCE_DATE_EPOCH environment variable not set,"
+                " setting zipinfo date to default=%s",
+                default,
+            )
+            return default
+        else:
+            if _env_date[0] < 1980:
+                logger.warning(
+                    "zipinfo date can't be earlier than 1980,"
+                    " setting zipinfo date to default=%s",
+                    default,
+                )
+                return default
+            return _env_date
+
     def _write_entry_points(self, fp: TextIO) -> None:
         """
         Write entry_points.txt.
