@@ -7,6 +7,8 @@ from poetry.core.constraints.version import VersionConstraint
 from poetry.core.constraints.version import VersionRange
 from poetry.core.constraints.version import VersionUnion
 from poetry.core.constraints.version import parse_constraint
+from poetry.core.constraints.version import parse_marker_version_constraint
+from poetry.core.constraints.version.exceptions import ParseConstraintError
 from poetry.core.version.pep440 import ReleaseTag
 
 
@@ -16,8 +18,6 @@ from poetry.core.version.pep440 import ReleaseTag
         ("*", VersionRange()),
         ("*.*", VersionRange()),
         ("v*.*", VersionRange()),
-        ("*.x.*", VersionRange()),
-        ("x.X.x.*", VersionRange()),
         (">1.0.0", VersionRange(min=Version.from_parts(1, 0, 0))),
         ("<1.2.3", VersionRange(max=Version.from_parts(1, 2, 3))),
         ("<=1.2.3", VersionRange(max=Version.from_parts(1, 2, 3), include_max=True)),
@@ -66,21 +66,8 @@ def test_parse_constraint(input: str, constraint: Version | VersionRange) -> Non
             "2.0.*",
             VersionRange(Version.parse("2.0.dev0"), Version.parse("2.1.dev0"), True),
         ),
-        (
-            "2.x",
-            VersionRange(Version.parse("2.dev0"), Version.parse("3.dev0"), True),
-        ),
-        (
-            "2.x.x",
-            VersionRange(Version.parse("2.dev0"), Version.parse("3.dev0"), True),
-        ),
-        (
-            "2.2.X",
-            VersionRange(Version.parse("2.2.dev0"), Version.parse("2.3.dev0"), True),
-        ),
         ("0.*", VersionRange(Version.parse("0.dev0"), Version.parse("1.dev0"), True)),
         ("0.*.*", VersionRange(Version.parse("0.dev0"), Version.parse("1.dev0"), True)),
-        ("0.x", VersionRange(Version.parse("0.dev0"), Version.parse("1.dev0"), True)),
         (
             "2.0.post1.*",
             VersionRange(
@@ -595,3 +582,14 @@ def test_parse_constraint_with_white_space_padding(
     padding = " " * (4 if with_whitespace_padding else 0)
     constraint = padding.join(["", *constraint_parts, ""])
     assert parse_constraint(constraint) == expected
+
+
+def test_parse_marker_constraint_does_not_allow_invalid_version() -> None:
+    with pytest.raises(ParseConstraintError):
+        parse_marker_version_constraint("4.9.253-tegra")
+
+
+def test_parse_marker_constraint_does_allow_invalid_version_if_requested() -> None:
+    assert parse_marker_version_constraint(
+        "4.9.253-tegra", pep440=False
+    ) == Version.from_parts(4, 9, 253, local="tegra")
