@@ -39,7 +39,7 @@ def create_dependency(
     if source_name:
         dep.source_name = source_name
     if marker:
-        dep.marker = marker  # type: ignore[assignment]
+        dep.marker = marker
     return dep
 
 
@@ -349,6 +349,40 @@ def test_remove_dependency_removes_from_both_lists() -> None:
         ),
         # extras - special
         # root extras do not have an extra marker, they just have set _in_extras!
+        (
+            [
+                create_dependency(
+                    "foo", source_name="src", optional=True, in_extras=("extra1",)
+                )
+            ],
+            [create_dependency("foo", source_name="src", optional=True)],
+            [
+                create_dependency(
+                    "foo", source_name="src", optional=True, in_extras=("extra1",)
+                )
+            ],
+        ),
+        (
+            [
+                create_dependency(
+                    "foo", source_name="src", optional=True, in_extras=("extra1",)
+                )
+            ],
+            [create_dependency("foo", source_name="src")],
+            (
+                [
+                    create_dependency(
+                        "foo", source_name="src", optional=True, in_extras=("extra1",)
+                    )
+                ],
+                [
+                    create_dependency(
+                        "foo", source_name="src", optional=True, in_extras=("extra1",)
+                    ),
+                    create_dependency("foo", source_name="src"),
+                ],
+            ),
+        ),
         (
             [
                 Dependency.create_from_pep_508("foo;extra!='extra1'"),
@@ -463,6 +497,9 @@ def test_dependencies_for_locking(
     assert [d.in_extras for d in group.dependencies_for_locking] == [
         d.in_extras for d in expected_dependencies
     ]
+    assert [d.is_optional() for d in group.dependencies_for_locking] == [
+        d.is_optional() for d in expected_dependencies
+    ]
 
 
 @pytest.mark.parametrize(
@@ -491,3 +528,119 @@ def test_dependencies_for_locking_failure(
 
     with pytest.raises(ValueError):
         _ = group.dependencies_for_locking
+
+
+@pytest.mark.parametrize(
+    ["pretty_name", "canonicalized_name"],
+    [
+        ("Group", "group"),
+        ("foo_bar", "foo-bar"),
+        ("Foo-Bar", "foo-bar"),
+    ],
+)
+def test_dependency_group_use_canonicalize_name(
+    pretty_name: str, canonicalized_name: str
+) -> None:
+    group = DependencyGroup(pretty_name)
+    assert group.name == canonicalized_name
+    assert group.pretty_name == pretty_name
+
+
+def test_include_dependency_groups() -> None:
+    group = DependencyGroup(name="group")
+    group.add_poetry_dependency(
+        Dependency(name="foo", constraint="*", groups=["group"])
+    )
+
+    group_2 = DependencyGroup(name="group2")
+    group_2.add_poetry_dependency(
+        Dependency(name="bar", constraint="*", groups=["group2"])
+    )
+
+    # This will resolve to a more precise constraint in dependencies_for_locking.
+    group_3 = DependencyGroup(name="group3")
+    group_3._dependencies = [Dependency(name="baz", constraint="<2", groups=["group3"])]
+    group_3._poetry_dependencies = [
+        Dependency(name="baz", constraint=">=1", groups=["group3"])
+    ]
+
+    group.include_dependency_group(group_2)
+    group.include_dependency_group(group_3)
+
+    assert [dep.name for dep in group.dependencies] == ["foo", "bar", "baz"]
+    assert [dep.name for dep in group.dependencies_for_locking] == ["foo", "bar", "baz"]
+    assert [dep.pretty_constraint for dep in group.dependencies] == ["*", "*", "<2"]
+    assert [dep.pretty_constraint for dep in group.dependencies_for_locking] == [
+        "*",
+        "*",
+        ">=1,<2",
+    ]
+    for dep in group.dependencies:
+        assert dep.groups == {"group"}
+
+    assert [dep.name for dep in group_2.dependencies] == ["bar"]
+    assert [dep.name for dep in group_2.dependencies_for_locking] == ["bar"]
+    for dep in group_2.dependencies:
+        assert dep.groups == {"group2"}
+
+    assert [dep.name for dep in group_3.dependencies] == ["baz"]
+    assert [dep.name for dep in group_3.dependencies_for_locking] == ["baz"]
+    assert [dep.pretty_constraint for dep in group_3.dependencies] == ["<2"]
+    assert [dep.pretty_constraint for dep in group_3.dependencies_for_locking] == [
+        ">=1,<2"
+    ]
+    for dep in group_3.dependencies:
+        assert dep.groups == {"group3"}
+
+
+def test_include_empty_dependency_group() -> None:
+    group_main = DependencyGroup(name="group")
+    group_main.add_poetry_dependency(
+        Dependency(name="foo", constraint="*", groups=["group"])
+    )
+
+    empty_group = DependencyGroup(name="empty")
+    # Include empty dependency group
+    group_main.include_dependency_group(empty_group)
+
+    # Assert that including an empty group does not affect group_main's dependencies.
+    assert [dep.name for dep in group_main.dependencies] == ["foo"]
+    assert [dep.name for dep in group_main.dependencies_for_locking] == ["foo"]
+
+
+@pytest.mark.parametrize("group_name", ["group_2", "Group-2", "group-2"])
+def test_include_dependency_group_raise_if_including_itself(group_name: str) -> None:
+    group = DependencyGroup(name="group-2")
+    group.add_poetry_dependency(
+        Dependency(name="foo", constraint="*", groups=["group"])
+    )
+
+    with pytest.raises(
+        ValueError, match="Cannot include the dependency group to itself"
+    ):
+        group.include_dependency_group(DependencyGroup(name=group_name))
+
+
+@pytest.mark.parametrize("group_name", ["group_2", "Group-2", "group-2"])
+def test_include_dependency_group_raise_if_already_included(group_name: str) -> None:
+    group = DependencyGroup(name="group")
+    group.add_poetry_dependency(
+        Dependency(name="foo", constraint="*", groups=["group"])
+    )
+
+    group_2 = DependencyGroup(name="group_2")
+    group_2.add_poetry_dependency(
+        Dependency(name="bar", constraint="*", groups=["group2"])
+    )
+
+    group.include_dependency_group(group_2)
+
+    group_3 = DependencyGroup(name=group_name)
+    group_3.add_poetry_dependency(
+        Dependency(name="bar", constraint="*", groups=["group2"])
+    )
+
+    with pytest.raises(
+        ValueError, match=f"Dependency group {group_name} is already included"
+    ):
+        group.include_dependency_group(group_3)
